<?php

/**
 * @file
 * Functions needed for refresh elements list(statuses, categories).
 */

namespace Drupal\desk_net;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\desk_net\Controller\ActionController;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\field\Entity\FieldConfig;
use Drupal\desk_net\Controller\ModuleSettings;
use Drupal\desk_net\Controller\RequestsController;
use Drupal\desk_net\Collection\NoticesCollection;

class DeleteMethods extends ControllerBase {

  /**
   * Perform shape list of deleted items.
   *
   * @param string $setting_name
   *   The name of the setting in the configuration.
   * @param array $new_items_list
   *   The new element list.
   * @param array $save_items_list
   *   The save element list in Drupal Configuration Settings.
   * @param array $drupal_items_list
   *   The basic Drupal elements for Node.
   *   array[key] string
   *   The key has ID or Slug element value.
   * @param string $type
   *   The type elements in list.
   */
  public static function shapeDeletedItems($setting_name, array $new_items_list, array $save_items_list,
    array $drupal_items_list, $type, $clear_both_direction = TRUE
  ) {
    $delete_element_list = [];

    if (!empty($save_items_list)) {
      foreach ($save_items_list as $value) {
        $in_stock = FALSE;
        foreach ($new_items_list as $content) {
          if ($value["id"] == $content["id"]) {
            $in_stock = TRUE;
          }
        }

        if (!$in_stock) {
          $delete_element_list[] = $value['id'];
        }
      }
    }

    if (!empty($delete_element_list)) {
      $delete_element_list = DeleteMethods::checkSubItems($delete_element_list, $save_items_list);
      DeleteMethods::deleteItems($setting_name, $delete_element_list, $drupal_items_list, $type, $clear_both_direction);
    }
  }

  /**
   * Perform checking parent elements in the list of elements to be deleted.
   *
   * @param array $delete_element_list
   *   The element for delete from saved list.
   * @param array $save_items_list
   *   The save element list in Drupal DB.
   *
   * @return array
   *   The list with deleting elements.
   */
  private static function checkSubItems(array $delete_element_list, array $save_items_list) {
    $under_sub_category = [];

    if (!empty($save_items_list)) {
      foreach ($save_items_list as $item) {
        foreach ($delete_element_list as $value) {
          if (isset($item["category"]) && $value == $item["category"]) {
            $delete_element_list[] = $item['id'];
            $under_sub_category[] = $item['id'];
          }
        }
      }
    }

    if (!empty($under_sub_category)) {
      return DeleteMethods::checkSubItems($under_sub_category, $save_items_list);
    }
    else {
      return $delete_element_list;
    }
  }

  /**
   * Perform deleted items and mapping category.
   *
   * @param string $setting_name
   *   The name of the setting in the configuration.
   * @param array $delete_element_list
   *   The element for delete from saved list.
   * @param array $drupal_items_list
   *   The basic Drupal elements for Node.
   * @param string $type
   *   The type elements in list.
   * @param bool $clear_both_direction
   *   A boolean indicating whether to clear in both directions.
   */
  private static function deleteItems($setting_name, array $delete_element_list, array $drupal_items_list, $type, $clear_both_direction) {
    if (!empty($drupal_items_list) && !empty($delete_element_list)) {
      $value_list = \Drupal::config('desk_net.settings')->get($setting_name);

      if ($value_list !== NULL) {
        $value_list = unserialize($value_list);
      }

      foreach ($delete_element_list as $value) {
        foreach ($drupal_items_list as $key) {
          $field_name = ActionController::generateFieldMatchingName($type, 'drupal_to_desk_net', $key);
          if (isset($value_list[$field_name]) && $value == $value_list[$field_name] && $clear_both_direction) {
            $value_list[$field_name] = 'no_' . $type;
          }
        }
        $field_name = ActionController::generateFieldMatchingName($type, 'desk_net_to_drupal', $value);
        unset($value_list[$field_name]);
      }

      \Drupal::configFactory()
        ->getEditable('desk_net.settings')
        ->set($setting_name, serialize($value_list))
        ->save();
    }
  }

  /**
   * Deleting all Desk-Net fields for Content-types.
   */
  public static function deleteDeskNetFields() {
    $load_content_types = \Drupal::service('entity_type.manager')
      ->getStorage('node_type')
      ->loadMultiple();

    foreach ($load_content_types as $content_type) {
      DeleteMethods::deleteCustomField('desk_net_' . $content_type->id() . '_revision', 'node');
    }
    // Delete Desk-Net user id.
    DeleteMethods::deleteCustomField(ModuleSettings::variableGet('desk_net_author_id'), 'user', 'by_name');
  }

  /**
   * Deleting Desk-Net module fields.
   *
   * @param string $field_name
   *   The field name.
   * @param string $entity_type
   *   The entity type.
   * @param string $delete_by
   *   The type deleting element.
   */
  private static function deleteCustomField($field_name, $entity_type = 'node', $delete_by = 'by_hash') {
    switch ($delete_by) {
      case 'by_name':
        if (FieldStorageConfig::loadByName($entity_type, $field_name) !== NULL) {
          FieldStorageConfig::loadByName($entity_type, $field_name)->delete();
        }
        break;

      default:
        // Getting hash name for field.
        $hash_field = ModuleSettings::variableGet($field_name);
        // Deleting matching field.
        ModuleSettings::variableDel($field_name);

        // Deleting field.
        if ($hash_field !== NULL && FieldStorageConfig::loadByName($entity_type, $hash_field) !== NULL) {
          FieldStorageConfig::loadByName($entity_type, $hash_field)->delete();
        }
    }
  }

  /**
   * Deleting node in Drupal 9.
   *
   * @param EntityInterface $entity
   *   The node that is being deleted.
   *
   * @return bool
   *   The result of deleting article in Drupal.
   */
  public function deleteElement(EntityInterface $entity) {
    // Deleting variable date for article.
    if (ModuleSettings::variableGet("desk-net-node-" . $entity->vid->value . "-date") !== NULL) {
      ModuleSettings::variableDel("desk-net-node-" . $entity->vid->value . "-date");
    }
    // Getting Desk-Net revision data.
    $desk_net_revision = ModuleSettings::deskNetRevisionGet($entity);

    if (isset($desk_net_revision['desk_net_story_id'])) {
      // Check Deleted/Removed Status.
      if (!empty($desk_net_revision['desk_net_removed_status']) && $desk_net_revision['desk_net_removed_status'] === 'desk_net_removed') {
        return FALSE;
      }
      // Check exist story on Desk-Net side.
      $story_data = (new RequestsController())->get(ModuleSettings::DN_BASE_URL, 'elements', $desk_net_revision['desk_net_story_id']);

      if ($story_data === 'unauthorized') {
        \Drupal::messenger()->addError(NoticesCollection::getNotice(3));

        return FALSE;
      }

      $story_data = json_decode($story_data, TRUE);

      if ($story_data === 'not_show_new_notice' || empty($story_data) || !empty($story_data['message'])) {
        return FALSE;
      }

      if (isset($desk_net_revision['desk_net_publications_id']) && !empty($desk_net_revision['desk_net_publications_id'])) {
        if (ModuleSettings::checkUserCredentials()) {
          (new RequestsController())->customRequest('DELETE', [], ModuleSettings::DN_BASE_URL, 'elements/publication', $desk_net_revision['desk_net_publications_id']);
        }
        else {
          \Drupal::messenger()->addError(NoticesCollection::getNotice(5));
        }
      }
    }
  }

}