<?php

/**
 * @file
 * The "Content Type Matching" form.
 */

namespace Drupal\desk_net\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

use Drupal\desk_net\Controller\ModuleSettings;
use Drupal\desk_net\Collection\NoticesCollection;
use Drupal\desk_net\PageTemplate\PageTemplate;
use Drupal\desk_net\Controller\RequestsController;
use Drupal\desk_net\DeleteMethods;

class DeskNetContentTypeMatching extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'desk_net_content_type_matching';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Loading all Drupal Content types from Desk-Net.
    $is_content_types_exist = $this->getTypes();
    $config = \Drupal::config('desk_net.settings');
    $configuration_setting_name = 'desk_net_selected_content_type_matching_list';
    $types_for_platform = $config->get('desk_net_types');

    if ($types_for_platform !== NULL) {
      $types_for_platform = unserialize($types_for_platform);
    }

    if (!empty($config->get('platform_id'))) {
      $html = '<h2>' . \Drupal::translation()
          ->translate('Content Type Matching') . '</h2>';
      $html .= '<p>';
      $html .= \Drupal::translation()
        ->translate('Use this page to match Types in Desk-Net to content types in Drupal.');
      $html .= '</p>';

      if ($is_content_types_exist && !empty(ModuleSettings::variableGet('desk_net_token'))) {
        $do_not_import = [
          'do_not_import' => [
            'id' => 'do_not_import',
            'name' => 'Do not import',
          ],
        ];
        $drupal_to_desk_net = [];
        // Load all Drupal Content types.
        $load_content_types = \Drupal::service('entity_type.manager')
          ->getStorage('node_type')
          ->loadMultiple();

        if (!empty($load_content_types)) {
          foreach ($load_content_types as $content_type) {
            $drupal_to_desk_net[$content_type->id()]['id'] = $content_type->id();
            $drupal_to_desk_net[$content_type->id()]['name'] = $content_type->label();

            // Creating custom Desk-Net fields for this content type if they don't exist.
            ModuleSettings::createCustomFields($content_type->id());
          }
        }

        // Adding default menu item 'No category'.
        $desk_net_to_drupal['no_type']['id'] = 'no_type';
        $desk_net_to_drupal['no_type']['name'] = 'No Type';
        $drupal_to_desk_net = array_merge($do_not_import, $drupal_to_desk_net);

        if (!empty($types_for_platform)) {
          foreach ($types_for_platform as $value) {
            $desk_net_to_drupal[$value['id']]['id'] = $value['id'];
            $desk_net_to_drupal[$value['id']]['name'] = $value['name'];
          }
        }

        $html .= '<h4 class="dn_b-title">' . \Drupal::translation()
            ->translate('Desk-Net to Drupal') . '</h4>';

        $form['html'] = [
          '#markup' => $html,
        ];

        $form['desk_net_to_drupal_matching'] = PageTemplate::desk_net_matching_page_template(
          $configuration_setting_name, $desk_net_to_drupal, $drupal_to_desk_net,
          'type', 'desk_net_to_drupal', 'article');

        $sub_title = '<h4 class="dn_b-title">' . \Drupal::translation()
            ->translate('Drupal to Desk-Net') . '</h4>';

        $form['subTitle'] = [
          '#markup' => $sub_title,
        ];

        unset($drupal_to_desk_net['do_not_import']);
        $desk_net_to_drupal = array_merge($do_not_import, $desk_net_to_drupal);

        $form['drupal_to_desk_net_matching'] = PageTemplate::desk_net_matching_page_template(
          $configuration_setting_name, $drupal_to_desk_net, $desk_net_to_drupal,
          'type', 'drupal_to_desk_net', 'no_type');

        $form['submit'] = [
          '#type' => 'submit',
          '#value' => \Drupal::translation()->translate('Save'),
        ];

        return $form;
      }
      elseif (!$is_content_types_exist && empty($types_for_platform)) {
        $html .= '<p>';
        $html .= '<strong>' . \Drupal::translation()
            ->translate('Please note:') . '</strong> ';
        $html .= \Drupal::translation()
          ->translate('The synchronized platform in Desk-Net lacks platform types for matching.');
        $html .= '</p>';

        $form['html'] = [
          '#markup' => $html,
        ];

        return $form;
      }
      else {
        \Drupal::messenger()->addError(NoticesCollection::getNotice(9));
      }
    }
    else {
      \Drupal::messenger()->addError(NoticesCollection::getNotice(10));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $formValues = NULL;

    if (!empty($form_state->getValues())) {
      foreach ($form_state->getValues() as $key => $value) {
        if ($key != 'form_id' && $key != 'op' && $key != 'form_token' &&
          $key != 'form_build_id' && $key != 'submit'
        ) {
          $formValues[$key] = $value;
        }
      }

      // Save the values to configuration.
      \Drupal::configFactory()
        ->getEditable('desk_net.settings')
        ->set('desk_net_selected_content_type_matching_list', serialize($formValues))
        ->save();

      \Drupal::messenger()->addStatus(NoticesCollection::getNotice(13));
    }
  }

  /**
   * Perform get Types for Desk-Net platform.
   *
   * @return bool
   *   The result loading types list from Desk-Net.
   */
  private function getTypes() {
    $config = \Drupal::config('desk_net.settings');
    $configFactory = \Drupal::configFactory()->getEditable('desk_net.settings');
    $platform_id = $config->get('platform_id');
    $saved_types_list_for_platform = $config->get('desk_net_types');

    if ($saved_types_list_for_platform !== NULL) {
      $saved_types_list_for_platform = unserialize($saved_types_list_for_platform);
    }

    if (!empty($platform_id)) {
      $new_types_list = (new RequestsController())->get(ModuleSettings::DN_BASE_URL,
        'types/platform', $platform_id);

      if ($new_types_list === 'unauthorized') {
        \Drupal::messenger()->addError(NoticesCollection::getNotice(3));

        return FALSE;
      }

      $new_types_list = json_decode($new_types_list, TRUE);

      if (empty($new_types_list)) {
        $configFactory->set('desk_net_types', serialize($new_types_list))
          ->save();
        return FALSE;
      }

      if (!empty($new_types_list['message']) || $new_types_list === 'not_show_new_notice') {
        return FALSE;
      }

      if (!empty($saved_types_list_for_platform)) {
        // Load all Drupal Content types.
        $load_content_types = \Drupal::service('entity_type.manager')
          ->getStorage('node_type')
          ->loadMultiple();

        $content_types_list = [];

        if (!empty($load_content_types)) {
          foreach ($load_content_types as $content_type) {
            $content_types_list[] = $content_type->id();
          }
        }

        DeleteMethods::shapeDeletedItems('desk_net_selected_content_type_matching_list', $new_types_list,
          $saved_types_list_for_platform, $content_types_list, 'type');
      }

      $configFactory->set('desk_net_types', serialize($new_types_list))->save();

      return TRUE;
    }

    return FALSE;
  }

}