<?php

/**
 * @file
 * The "Task Matching" form.
 */

namespace Drupal\desk_net\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

use Drupal\desk_net\Controller\ModuleSettings;
use Drupal\desk_net\Collection\NoticesCollection;
use Drupal\desk_net\PageTemplate\PageTemplate;
use Drupal\desk_net\Controller\RequestsController;

class DeskNetTaskMatching extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'desk_net_task_matching';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $configuration_setting_name = 'desk_net_selected_task_matching_list';
    $desk_net_task_list = $this->getTasks();
    $this->clearFormState($desk_net_task_list);

    if (!empty(\Drupal::config('desk_net.settings')->get('platform_id'))) {
      if ($desk_net_task_list && !empty(ModuleSettings::variableGet('desk_net_token'))) {
        $html = '<h2>' . \Drupal::translation()
            ->translate('Tasks Matching') . '</h2>';
        $html .= '<p>';
        $html .= \Drupal::translation()->translate('Use this page to match the 
        default task format in Desk-Net on creating a Story from Drupal to Desk-Net.');
        $html .= '</p>';
        $html .= '<h4 class="dn_b-title">' . \Drupal::translation()
            ->translate('Drupal to Desk-Net') . '</h4>';

        $form['html'] = [
          '#markup' => $html,
        ];

        $desk_net_to_drupal = [
          'default_task_in_desk_net' => [
            'id' => 'default_task_in_desk_net',
            'name' => 'Default Task in Desk-Net',
          ],
        ];

        foreach ($desk_net_task_list as $task) {
          $drupal_to_desk_net[$task['id']]['id'] = $task['id'];
          $drupal_to_desk_net[$task['id']]['name'] = $task['name'];
        }

        $form['desk_net_to_drupal_matching'] = PageTemplate::desk_net_matching_page_template(
          $configuration_setting_name, $desk_net_to_drupal, $drupal_to_desk_net,
          'task', '', '18');

        $form['submit'] = [
          '#type' => 'submit',
          '#value' => \Drupal::translation()->translate('Save'),
        ];

        return $form;
      }
      else {
        \Drupal::messenger()->addError(NoticesCollection::getNotice(9));
      }
    }
    else {
      \Drupal::messenger()->addError(NoticesCollection::getNotice(10));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $formValues = NULL;

    if (!empty($form_state->getValues())) {
      foreach ($form_state->getValues() as $key => $value) {
        if ($key != 'form_id' && $key != 'op' && $key != 'form_token' &&
          $key != 'form_build_id' && $key != 'submit'
        ) {
          $formValues[$key] = $value;
        }
      }

      // Save the values to configuration.
      \Drupal::configFactory()
        ->getEditable('desk_net.settings')
        ->set('desk_net_selected_task_matching_list', serialize($formValues))
        ->save();

      \Drupal::messenger()->addStatus(NoticesCollection::getNotice(13));
    }
  }

  /**
   * Retrieves tasks from the Desk-Net platform.
   *
   * @return bool|array
   *   The result of loading the tasks list from Desk-Net, or FALSE if
   *   unsuccessful.
   */
  private function getTasks() {
    $config = \Drupal::config('desk_net.settings');
    $platform_id = $config->get('platform_id');
    $saved_task_list_for_platform = $config->get('desk_net_task_list');

    if ($saved_task_list_for_platform !== NULL) {
      $saved_task_list_for_platform = unserialize($saved_task_list_for_platform);
    }

    if (!empty($platform_id)) {
      $task_list_for_platform = (new RequestsController())->get(ModuleSettings::DN_BASE_URL,
        'formats');

      if ($task_list_for_platform === 'unauthorized') {
        \Drupal::messenger()->addError(NoticesCollection::getNotice(9));

        return $saved_task_list_for_platform;
      }

      $task_list_for_platform = json_decode($task_list_for_platform, TRUE);

      if (!empty($task_list_for_platform['message']) || $task_list_for_platform === 'not_show_new_notice'
        || empty($task_list_for_platform)) {
        return $saved_task_list_for_platform;
      }

      \Drupal::configFactory()
        ->getEditable('desk_net.settings')
        ->set('desk_net_task_list', serialize($task_list_for_platform))
        ->save();

      return $task_list_for_platform;
    }
  }

  /**
   * Checks and updates the default task in the Desk-Net form state.
   *
   * @param array $desk_net_task_list
   *   The list of tasks retrieved from Desk-Net.
   */
  private function clearFormState($desk_net_task_list) {
    if (empty($desk_net_task_list)) {
      return;
    }

    if (empty($desk_net_task_list['desk_net_task__default_task_in_desk_net'])) {
      $desk_net_task_list['desk_net_task__default_task_in_desk_net'] = '18';

      \Drupal::configFactory()
        ->getEditable('desk_net.settings')
        ->set('desk_net_task_list', serialize($desk_net_task_list))
        ->save();
    }
  }

}